function GearEvaluator:AddEvaluatorFunction(name)
  if (not self.db.profile.evaluators[name]) then
    self:Print("Adding evaluator "..name.."!")

    self.db.profile.evaluators[name] = "return 0"
    self.evaluators[name] = self:CompileEvaluator(self.db.profile.evaluators[name])
    self:UpdateScore(name)

    self:UpdateEvaluatorConfigTable()
  else
    self:Print("Evaluator "..name.." is already defined!")
  end
end

function GearEvaluator:DeleteEvaluatorFunction(name)
  self:Print("Deleting evaluator "..name.."!")

  self.db.profile.evaluators[name] = nil
  self.evaluators[name] = nil
  self.scores[name] = nil

  self:UpdateEvaluatorConfigTable()
end

function GearEvaluator:UpdateEvaluatorFunction(name, func)
  self.db.profile.evaluators[name] = func
  self.evaluators[name] = self:CompileEvaluator(func, name)
  self:UpdateScore(name)
end

function GearEvaluator:CopyEvaluatorFunction(src, dst)
  self:Print("Copying evaluator function from "..src.." to "..dst)

  self.db.profile.evaluators[dst] = self.db.profile.evaluators[src]
  self.evaluators[dst] = self:CompileEvaluator(self.db.profile.evaluators[dst], dst)
  self:UpdateScore(dst)
end

function GearEvaluator:RenameEvaluatorFunction(name, newname)
  if (name == newname) then
    return
  end

  self:Print("Renaming "..name.." to "..newname)

  self.db.profile.evaluators[newname] = self.db.profile.evaluators[name]
  self.db.profile.evaluators[name] = nil

  self.evaluators[newname] = self:CompileEvaluator(self.db.profile.evaluators[newname], newname)

  self:UpdateEvaluatorConfigTable()
  self:UpdateScore(newname)
end

function GearEvaluator:GetEvaluatorFunction(name)
  return self.db.profile.evaluators[name]
end
