-- Initialization

GearEvaluator = LibStub("AceAddon-3.0"):NewAddon("GearEvaluator", "AceEvent-3.0", "AceConsole-3.0", "AceBucket-3.0")

local slotnames = {"HeadSlot",
  "NeckSlot",
  "ShoulderSlot",
  "BackSlot",
  "ChestSlot",
  "ShirtSlot",
  "TabardSlot",
  "WristSlot",
  "HandsSlot",
  "WaistSlot",
  "LegsSlot",
  "FeetSlot",
  "Finger0Slot",
  "Finger1Slot",
  "Trinket0Slot",
  "Trinket1Slot",
  "MainHandSlot",
  "SecondaryHandSlot",
  "RangedSlot",
  "AmmoSlot",
  --"Bag0Slot",
  --"Bag1Slot",
  --"Bag2Slot",
  --"Bag3Slot"
}

-- Functions

function GearEvaluator:Debug(level, message)
  if (self.db and self.db.profile and self.db.profile.debuglevel and self.db.profile.debuglevel >= level) then
    self:Print(message)
  end
end

function GearEvaluator:OnInitialize()
  local defaults = {
    profile = {
      debuglevel = 0,
      evaluators = {}
    }
  }

  if (not self.scores) then
    self.scores = {}
  end

  if (not self.errors) then
    self.errors = {}
  end

  if (not self.evaluators) then
    self.evaluators = {}
  end

  self.db = LibStub("AceDB-3.0"):New("GearEvaluatorDB", defaults)

  self:InitConfig()
  GearEvaluator.Broker:Init()
end

function GearEvaluator:OnEnable()
  self.invChangeBucket = self:RegisterBucketEvent("UNIT_INVENTORY_CHANGED", 0.1, "InventoryChangedCB")

  self:CompileAllEvaluators()
  self:UpdateScores()
end

function GearEvaluator:OnDisable()
  self:UnregisterBucket(self.invChangeBucket)
end

function GearEvaluator:CompileAllEvaluators()
  if (self.db.profile.evaluators) then
    for k, v in pairs(self.db.profile.evaluators) do
      self.evaluators[k] = self:CompileEvaluator(v, k)
    end
  end
end

function GearEvaluator:CompileEvaluator(text, name)
  local chunk, error = loadstring("return function(gear) "..text.." end", name)

  if (not chunk) then
    self:Print(error)
    self.errors[name] = error
  end

  return chunk()
end

function GearEvaluator:GetScore(evalname, gear)
  if (not self.evaluators[evalname]) then
    return nil, "Undefined evaluator '"..evalname.."'"
  else
    local func = self.evaluators[evalname]
    return pcall(func, gear)
  end
end

function GearEvaluator:UpdateScore(name)
  local success, score = self:GetScore(name, self:GetCurrentEquipment())

  if (not success) then
    self:Print(score)
    self.scores[name] = nil
    self.errors[name] = score
  else
    self.scores[name] = score
  end
end

function GearEvaluator:UpdateScores()
  for k, v in pairs(self.evaluators) do
    self:UpdateScore(k)
  end
end

function GearEvaluator:GetCurrentEquipment()
  local gear = {}

  for k, s in pairs(slotnames) do
    gear[s] = GetInventoryItemLink("player", GetInventorySlotInfo(s))
  end

  return gear
end

function GearEvaluator:GetActiveSlotNames()
  return slotnames
end

-- Event handlers

function GearEvaluator:InventoryChangedCB(units)
  if (units.player) then
    self:UpdateScores()
  end
end

-- Variable initalization

GearEvaluator:SetDefaultModuleLibraries("AceConsole-3.0")
GearEvaluator:SetDefaultModulePrototype({ Debug = GearEvaluator.Debug })
